// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <vector>
#include <cassert>
#include <storage/fsnode.h>

#include "main.h"
#include "appwindow.h"
#include "messages.h"
#include "searchtab.h"
#include "browsertab.h"
#include "searchcriteria.h"

using namespace os;
using namespace std;

AppWindow :: AppWindow( const Rect& cFrame ) : Window( cFrame, "main_window", "EFileBrowser" )
{	
	String start_path = getenv( "HOME" );

	// Sme start up values
	m_DiaFavouritesAdd = NULL;
	m_DiaFavouritesAdmin = NULL;

	// This resource get used at various points during the constructor to load icons etc.
	Resources cRes( get_image_id() );

	// Create menu
	m_MenuBar = new Menu( Rect(), "", ITEMS_IN_ROW );		
	AddChild( m_MenuBar );
	
	// Application menu
	m_ApplicationMenu = new Menu( Rect(), "Application", ITEMS_IN_COLUMN );
	m_ApplicationMenu->AddItem( "Quit", new Message( MSG_MENU_APP_QUIT ), "Ctrl + Q" );
	m_ApplicationMenu->AddItem( new MenuSeparator() );
	m_ApplicationMenu->AddItem( "About", new Message( MSG_MENU_APP_ABOUT ) );
	m_MenuBar->AddItem( m_ApplicationMenu );

	// Sidebar submenu
	m_SidebarMenu = new Menu(Rect(0,0,1,1), "Sidebars", ITEMS_IN_COLUMN);
	m_ViewMenu_SideBarTreeView = new CheckMenu( "Tree view", new Message( MSG_MENU_SIDEBAR_TREEVIEW ) );
	m_SidebarMenu->AddItem( m_ViewMenu_SideBarTreeView );
	m_ViewMenu_SideBarFileInfo = new CheckMenu( "File info", new Message( MSG_MENU_SIDEBAR_FILEINFO ) );
	m_SidebarMenu->AddItem( m_ViewMenu_SideBarFileInfo );
	m_ViewMenu_SideBarAttributes = new CheckMenu( "Attributes", new Message( MSG_MENU_SIDEBAR_ATTRIBUTES ) );
	m_SidebarMenu->AddItem( m_ViewMenu_SideBarAttributes );
	m_ViewMenu_SideBarSearch = new CheckMenu( "Search", new Message( MSG_MENU_SIDEBAR_SEARCH ) );
	m_SidebarMenu->AddItem( m_ViewMenu_SideBarSearch );
	m_ViewMenu_SideBarFavourites = new CheckMenu( "Favourites", new Message( MSG_MENU_SIDEBAR_FAVOURITES ) );
	m_SidebarMenu->AddItem( m_ViewMenu_SideBarFavourites );
	m_SidebarMenu->AddItem( new MenuSeparator() );
	m_ViewMenu_SideBarHide = new CheckMenu( "Hide", new Message( MSG_MENU_SIDEBAR_HIDE ) );
	m_ViewMenu_SideBarHide->SetChecked( true );
	m_SidebarMenu->AddItem( m_ViewMenu_SideBarHide );

	// Addressbar submenu
	m_AddressBarMenu = new Menu(Rect(0,0,1,1), "Addressbar", ITEMS_IN_COLUMN);
	m_ViewMenu_AddressBar_Text = new CheckMenu( "Textual", new Message( MSG_MENU_VIEW_ADDRESSBAR_TEXT ) );
	m_ViewMenu_AddressBar_Text->SetChecked( true );
	m_AddressBarMenu->AddItem( m_ViewMenu_AddressBar_Text );
	m_ViewMenu_AddressBar_Button = new CheckMenu( "Graphical", new Message( MSG_MENU_VIEW_ADDRESSBAR_BUTTON ) );
	m_ViewMenu_AddressBar_Button->SetChecked( false );
	m_AddressBarMenu->AddItem( m_ViewMenu_AddressBar_Button );
	m_AddressBarMenu->AddItem( new MenuSeparator() );
	m_ViewMenu_AddressBar_Hide = new CheckMenu( "Hide", new Message( MSG_MENU_VIEW_ADDRESSBAR_HIDE ) );
	m_ViewMenu_AddressBar_Hide->SetChecked( false );
	m_AddressBarMenu->AddItem( m_ViewMenu_AddressBar_Hide );

	// Tab view submenu
	m_TabSubMenu = new Menu(Rect(0,0,1,1), "Tabs", ITEMS_IN_COLUMN);
	m_ViewMenu_TabShow = new CheckMenu( "Show", new Message( MSG_MENU_VIEW_TAB_SHOW ) );
	m_ViewMenu_TabShow->SetChecked( false );
	m_TabSubMenu->AddItem( m_ViewMenu_TabShow );
	m_ViewMenu_TabHide = new CheckMenu( "Hide", new Message( MSG_MENU_VIEW_TAB_HIDE ) );
	m_ViewMenu_TabHide->SetChecked( false );
	m_TabSubMenu->AddItem( m_ViewMenu_TabHide );
	m_TabSubMenu->AddItem( new MenuSeparator() );
	m_ViewMenu_TabAutomatic = new CheckMenu( "Automatic", new Message( MSG_MENU_VIEW_TAB_AUTOMATIC ) );
	m_ViewMenu_TabAutomatic->SetChecked( true );
	m_TabSubMenu->AddItem( m_ViewMenu_TabAutomatic );

	// View menu
	m_ViewMenu = new Menu( Rect(), "View", ITEMS_IN_COLUMN );
	m_ViewMenu_Menu = new CheckMenu( "Menu", new Message( MSG_MENU_VIEW_MENU ) );
	m_ViewMenu_Menu->SetChecked( true );
	m_ViewMenu->AddItem( m_ViewMenu_Menu );
	m_ViewMenu_ToolBar = new CheckMenu( "Toolbar", new Message( MSG_MENU_VIEW_TOOLBAR ) );
	m_ViewMenu_ToolBar->SetChecked( true );
	m_ViewMenu->AddItem( m_ViewMenu_ToolBar );
	m_ViewMenu->AddItem( m_AddressBarMenu );
	m_ViewMenu->AddItem( m_TabSubMenu );
	m_ViewMenu->AddItem( m_SidebarMenu );
	m_ViewMenu_StatusBar = new CheckMenu( "Statusbar", new Message( MSG_MENU_VIEW_STATUSBAR ) );
	m_ViewMenu_StatusBar->SetChecked( true );
	m_ViewMenu->AddItem( m_ViewMenu_StatusBar );
	m_ViewMenu->AddItem( new MenuSeparator() );

	m_ViewMenu_BrowserIcons = new CheckMenu( "Icons", new Message( MSG_MENU_VIEW_ICONS ) );
	m_ViewMenu_BrowserIcons->SetChecked( true );
	m_ViewMenu->AddItem( m_ViewMenu_BrowserIcons );
	m_ViewMenu_BrowserList = new CheckMenu( "List", new Message( MSG_MENU_VIEW_LIST ) );
	m_ViewMenu_BrowserList->SetChecked( false );
	m_ViewMenu->AddItem( m_ViewMenu_BrowserList );
	m_ViewMenu_BrowserDetails = new CheckMenu( "Details", new Message( MSG_MENU_VIEW_DETAILS ) );
	m_ViewMenu_BrowserDetails->SetChecked( false );
	m_ViewMenu->AddItem( m_ViewMenu_BrowserDetails );

	m_MenuBar->AddItem( m_ViewMenu );

	// Favourites menu
	m_FavouritesMenu = new Menu( Rect(), "Favourites", ITEMS_IN_COLUMN );
	m_FavouritesMenu_Add = new MenuItem( "Add...", new Message( MSG_MENU_FAVOURITES_ADD ) );
	m_FavouritesMenu->AddItem( m_FavouritesMenu_Add );
	m_FavouritesMenu->AddItem( "Administrate...", new Message( MSG_MENU_FAVOURITES_ADMIN ) );
	m_MenuBar->AddItem( m_FavouritesMenu );

	// Tabs menu
	m_TabsMenu = new Menu( Rect(), "Tabs", ITEMS_IN_COLUMN );
	m_TabsMenu->AddItem( new MenuItem( "New Browser tab", new Message( MSG_MENU_TABS_NEW_BROWSER_TAB ), "Ctrl+T" ) );
	m_TabsMenu->AddItem( new MenuSeparator() );
	m_TabsMenu->AddItem( new MenuItem( "Close Tab", new Message( MSG_MENU_TABS_CLOSE ), "Ctrl+W" ) );
	m_TabsMenu->AddItem( new MenuItem( "Close All others", new Message( MSG_MENU_TABS_CLOSE_ALL_OTHER ) ) );
	m_TabsMenu->AddItem( new MenuItem( "Close All", new Message( MSG_MENU_TABS_CLOSE_ALL ) ) );
	m_TabsMenu->AddItem( new MenuSeparator() );
	m_TabsMenu->AddItem( new MenuItem( "Previous Tab", new Message( MSG_MENU_TABS_PREVIOUS_TAB ), "Ctrl+." ) );
	m_TabsMenu->AddItem( new MenuItem( "Next Tab", new Message( MSG_MENU_TABS_NEXT_TAB ), "Ctrl+," ) );
	m_MenuBar->AddItem( m_TabsMenu );

	// Toolbar
	BitmapImage* toolbar_icon;

	m_ToolBar = new ToolBar( Rect(), "toolbar", CF_FOLLOW_LEFT | CF_FOLLOW_TOP | CF_FOLLOW_RIGHT );

	toolbar_icon = new BitmapImage();
	toolbar_icon->Load( cRes.GetResourceStream( "home.png" ) );
	m_ToolBar->AddButton( "home", "Home", toolbar_icon, new Message( MSG_TOOLBAR_HOME ) );

	m_ToolBar->AddSeparator( "" );

	toolbar_icon = new BitmapImage();
	toolbar_icon->Load( cRes.GetResourceStream( "back.png" ) );
	m_ToolBar->AddButton( "back", "Back", toolbar_icon, new Message( MSG_TOOLBAR_BACK ) );

	toolbar_icon = new BitmapImage();
	toolbar_icon->Load( cRes.GetResourceStream( "up.png" ) );
	m_ToolBar->AddButton( "up", "Up", toolbar_icon, new Message( MSG_TOOLBAR_UP ) );

	AddChild( m_ToolBar );

	// Address bars
	m_DirBar = new DirBar( Rect(), "dirbar", new Message( MSG_DIRBAR_CHANGE_DIR ), CF_FOLLOW_TOP | CF_FOLLOW_RIGHT | CF_FOLLOW_LEFT );
	m_DirBar->SetPath( start_path );
	m_DirBar->Hide();
	m_DirBar->SetTarget( this );
	AddChild( m_DirBar );

	m_DirTextBar = new DirTextBar( Rect(), "dirtextbar", new Message( MSG_DIRBAR_CHANGE_DIR ), CF_FOLLOW_TOP | CF_FOLLOW_RIGHT | CF_FOLLOW_LEFT );
	m_DirTextBar->SetPath( start_path );
	AddChild( m_DirTextBar );

	// Create the sidebars
	m_SideBarTreeView = new TreeViewSideBar( Rect(), "Tree view",  CF_FOLLOW_TOP | CF_FOLLOW_BOTTOM | CF_FOLLOW_LEFT );
	m_SideBarTreeView->SetPath( start_path );
	m_SideBarTreeView->Hide();
	AddChild( m_SideBarTreeView );

	m_SideBarFileInfo = new FileInfoSideBar( Rect(), "File information", CF_FOLLOW_TOP | CF_FOLLOW_BOTTOM | CF_FOLLOW_LEFT );
	m_SideBarFileInfo->Hide();
	AddChild( m_SideBarFileInfo );

	m_SideBarAttributes = new AttributesSideBar( Rect(), "Attributes", CF_FOLLOW_TOP | CF_FOLLOW_BOTTOM | CF_FOLLOW_LEFT );
	m_SideBarAttributes->Hide();
	AddChild( m_SideBarAttributes );

	m_SideBarSearch = new SearchSideBar( Rect(), "Search", CF_FOLLOW_TOP | CF_FOLLOW_BOTTOM | CF_FOLLOW_LEFT );
	m_SideBarSearch->Hide();
	AddChild( m_SideBarSearch );

	m_SideBarFavourites = new FavouritesSideBar( Rect(), "Favourites", CF_FOLLOW_TOP | CF_FOLLOW_BOTTOM | CF_FOLLOW_LEFT );
	m_SideBarFavourites->Hide();
	AddChild( m_SideBarFavourites );

	// No current sidebar
	m_CurrentSideBar = NULL;

	// Create the tab
	m_Tab = new TabView( Rect(), "", os::CF_FOLLOW_ALL );
	m_Tab->SetTabType( TabView::AUTOMATIC_TABS );
	m_Tab->SetMessage( new Message( MSG_TAB ) );
	m_Tab->SetTarget( this );
	AddChild( m_Tab );

	// Create default browser tab
	BrowserTab* def = new BrowserTab( Rect(), getenv( "HOME" ) );	
	m_Tab->SetSelection( m_Tab->AppendTab( "Browser", def ) );
	PopulateTabs();

	// Fix layout
	Layout( true );
}

AppWindow :: ~AppWindow()
{
//	delete m_DiaFavouritesAdd;
}

void AppWindow :: Layout( bool force )
{
	// Get size of the window
	Rect cBounds = GetBounds();

	// Calculate menu bar location 
	if( m_MenuBar->IsVisible() || force )
	{
		Rect cMenuFrame = cBounds;
		cMenuFrame.bottom = m_MenuBar->GetPreferredSize( true ).y - 1.0f;
		m_MenuBar->SetFrame( cMenuFrame );
		cBounds.top = cMenuFrame.bottom + 1.0f;
	}

	// Calculate toolbar location
	if( m_ToolBar->IsVisible() || force )
	{
		Rect cToolbarFrame = cBounds;
		cToolbarFrame.bottom = cBounds.top + m_ToolBar->GetPreferredSize( false ).y;
		m_ToolBar->SetFrame( cToolbarFrame );
		cBounds.top = cToolbarFrame.bottom + 1.0f;
	}

	// Calculate addressbar location
	if( m_DirBar->IsVisible() ) // || force )
	{
		Rect cDirBarFrame = cBounds;
		cDirBarFrame.bottom = cBounds.top + m_DirBar->GetPreferredSize( false ).y;
		m_DirBar->SetFrame( cDirBarFrame );
		cBounds.top = cDirBarFrame.bottom + 1.0f;
	}

	// Calculate addressbar location
	if( m_DirTextBar->IsVisible() || force )
	{
		Rect cDirTextBarFrame = cBounds;
		cDirTextBarFrame.bottom = cBounds.top + m_DirTextBar->GetPreferredSize( false ).y;
		m_DirTextBar->SetFrame( cDirTextBarFrame );
		cBounds.top = cDirTextBarFrame.bottom + 1.0f;
	}

	// Calculate sidebar location
	if( m_CurrentSideBar != NULL && ( m_CurrentSideBar->IsVisible() || force ) )
	{
		Rect cSidebarFrame = cBounds;
		cSidebarFrame.right = cSidebarFrame.left + m_CurrentSideBar->GetPreferredSize( false ).x;
		m_CurrentSideBar->SetFrame( cSidebarFrame );
		cBounds.left = cSidebarFrame.right + 1.0f;
	}

	// Calculate filebrowser location
	Rect cTabFrame = cBounds;
	m_Tab->SetFrame( cTabFrame );

	// Poopulate the favourites menu
	PopulateFavourites();

	m_MenuBar->SetTargetForItems( this );
}

void AppWindow :: HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )	//Get the message code from the message
	{		
		case MSG_MENU_APP_QUIT:
			Application::GetInstance()->PostMessage( M_QUIT );
			break;
		case MSG_MENU_APP_ABOUT:
			{
				String cAbout, cCUPS;

				cAbout  = "EFilebrowser, version 0.1";
				cAbout += String( "\n\n" );
				cAbout += "A bloated filebrowser for the Syllable OS";
				cAbout += String( "\n\n" );
				cAbout += "Copyright 2007 Jonas Jarvoll";
				cAbout += String( "\n\n" );
				cAbout += "Released under GPL.";

				Alert *pcAlert = new Alert( "About EFilebrowser", cAbout, 0, "OK", NULL );
				pcAlert->CenterInWindow( this );
				pcAlert->Go( new Invoker() );

			break;
		}
			break;
		case MSG_MENU_SIDEBAR_TREEVIEW:
			SetSideBar( m_SideBarTreeView );
			m_ViewMenu_SideBarTreeView->SetChecked( true );
			break;
		case MSG_MENU_SIDEBAR_FILEINFO:
			SetSideBar( m_SideBarFileInfo );
			m_ViewMenu_SideBarFileInfo->SetChecked( true );
			break;
		case MSG_MENU_SIDEBAR_ATTRIBUTES:
			SetSideBar( m_SideBarAttributes );
			m_ViewMenu_SideBarAttributes->SetChecked( true );
			break;
		case MSG_MENU_SIDEBAR_SEARCH:
			SetSideBar( m_SideBarSearch );
			m_ViewMenu_SideBarSearch->SetChecked( true );
			break;
		case MSG_MENU_SIDEBAR_FAVOURITES:
			SetSideBar( m_SideBarFavourites );
			m_ViewMenu_SideBarFavourites->SetChecked( true );
			break;
		case MSG_MENU_SIDEBAR_HIDE:
			SetSideBar( NULL );
			m_ViewMenu_SideBarHide->SetChecked( true );
			break;
		case MSG_MENU_VIEW_MENU:
			if( m_MenuBar->IsVisible() )
				m_MenuBar->Hide();
			else
				m_MenuBar->Show();
			Layout();
			break;
		case MSG_MENU_VIEW_TOOLBAR:
			if( m_ToolBar->IsVisible() )
				m_ToolBar->Hide();
			else
				m_ToolBar->Show();
			Layout();
			break;
		case MSG_MENU_VIEW_TAB_SHOW:
			m_ViewMenu_TabShow->SetChecked( true );
			m_ViewMenu_TabHide->SetChecked( false );
			m_ViewMenu_TabAutomatic->SetChecked( false );
			m_Tab->SetTabType( TabView::SHOW_TABS );
			break;
		case MSG_MENU_VIEW_TAB_HIDE:
			m_ViewMenu_TabShow->SetChecked( false );
			m_ViewMenu_TabHide->SetChecked( true );
			m_ViewMenu_TabAutomatic->SetChecked( false );
			m_Tab->SetTabType( TabView::HIDE_TABS );
			break;
		case MSG_MENU_VIEW_TAB_AUTOMATIC:
			m_ViewMenu_TabShow->SetChecked( false );
			m_ViewMenu_TabHide->SetChecked( false );
			m_ViewMenu_TabAutomatic->SetChecked( true );
			m_Tab->SetTabType( TabView::AUTOMATIC_TABS );
			break;
		case MSG_MENU_VIEW_ADDRESSBAR_TEXT:
			m_ViewMenu_AddressBar_Text->SetChecked( true );
			m_ViewMenu_AddressBar_Button->SetChecked( false );
			m_ViewMenu_AddressBar_Hide->SetChecked( false );
			if( !m_DirTextBar->IsVisible() )
				m_DirTextBar->Show( true );			
			if( m_DirBar->IsVisible() )
				m_DirBar->Hide();
			Layout();
			break;
		case MSG_MENU_VIEW_ADDRESSBAR_BUTTON:
			m_ViewMenu_AddressBar_Text->SetChecked( false );
			m_ViewMenu_AddressBar_Button->SetChecked( true );
			m_ViewMenu_AddressBar_Hide->SetChecked( false );
			if( m_DirTextBar->IsVisible() )
				m_DirTextBar->Hide();			
			if( ! m_DirBar->IsVisible() )
				m_DirBar->Show( true );
			Layout();			
			break;
		case MSG_MENU_VIEW_ADDRESSBAR_HIDE:
			m_ViewMenu_AddressBar_Text->SetChecked( false );
			m_ViewMenu_AddressBar_Button->SetChecked( false );
			m_ViewMenu_AddressBar_Hide->SetChecked( true );
			if( m_DirBar->IsVisible() )
				m_DirBar->Hide();
			if( m_DirTextBar->IsVisible() )
				m_DirTextBar->Hide();
			Layout();
			break;
		case MSG_MENU_VIEW_STATUSBAR:
			if( BrowserTab::IsStatusBarVisible() )
				 BrowserTab::SetStatusBarVisibility( false );
			else
				 BrowserTab::SetStatusBarVisibility( true );
			m_Tab->FrameSized( Point( 0, 0 ) );
			break;
		case MSG_MENU_VIEW_ICONS:
			m_ViewMenu_BrowserIcons->SetChecked( true );
			m_ViewMenu_BrowserList->SetChecked( false );
			m_ViewMenu_BrowserDetails->SetChecked( false );
			SetView( IconDirectoryView::VIEW_ICONS );
			break;
		case MSG_MENU_VIEW_LIST:
			m_ViewMenu_BrowserIcons->SetChecked( false );
			m_ViewMenu_BrowserList->SetChecked( true );
			m_ViewMenu_BrowserDetails->SetChecked( false );
			SetView( IconDirectoryView::VIEW_LIST );
			break;
		case MSG_MENU_VIEW_DETAILS:
			m_ViewMenu_BrowserIcons->SetChecked( false );
			m_ViewMenu_BrowserList->SetChecked( false );
			m_ViewMenu_BrowserDetails->SetChecked( true );
			SetView( IconDirectoryView::VIEW_DETAILS );
			break;

		// Favourites menu
		case MSG_MENU_FAVOURITES_ADD:
			ShowFavouritesAddDialog();
			break;
		case MSG_MENU_FAVOURITES_ADMIN:
			ShowFavouritesAdminDialog();
			break;

		// Tab menu
		case MSG_MENU_TABS_NEW_BROWSER_TAB:
			{
				// Extract the path
				String path;

				if( pcMessage->FindString( "file/path", &path.str() ) != 0 )
					path = getenv( "HOME" );
				m_Tab->SetSelection( m_Tab->AppendTab( "Browser", new BrowserTab( Rect(), path ) ), false );
				PopulateTabs();

				// Update the address bars
				m_DirBar->SetPath( path );
				m_DirTextBar->SetPath( path );
				UpdateStatusBar();
				break;
			}
		case MSG_MENU_TABS_CLOSE:
			CloseTab( MSG_MENU_TABS_CLOSE );
			break;
		case MSG_MENU_TABS_CLOSE_ALL_OTHER:
			CloseTab( MSG_MENU_TABS_CLOSE_ALL_OTHER );
			break;
		case MSG_MENU_TABS_CLOSE_ALL:
			CloseTab( MSG_MENU_TABS_CLOSE_ALL );
			break;
		case MSG_MENU_TABS_NEXT_TAB:
			CycleTab( MSG_MENU_TABS_NEXT_TAB );
			break;
		case MSG_MENU_TABS_PREVIOUS_TAB:
			CycleTab( MSG_MENU_TABS_PREVIOUS_TAB );
			break;
		case MSG_MENU_TABS_ACTIVATE_TAB:
			{
				int16 sel;
				if( pcMessage->FindInt16( "tab_no", &sel ) == 0  )
					SetTab( (int) sel, true );
			}
			break;

		// Toolbar messages
		case MSG_TOOLBAR_HOME:
			ToolbarHome();
			break;
		case MSG_TOOLBAR_BACK:
			ToolbarBack();
			break;
		case MSG_TOOLBAR_UP:
			ToolbarUp();
			break;

		// Tab
		case MSG_TAB:
			SetTab( m_Tab->GetSelection(), false );
			break;

		// Sidebar messages
		case MSG_SIDEBAR_SEARCH:
			Search( pcMessage );
			break;
		case MSG_SIDEBAR_FAVOURITES_ADD:
			AddQuickFavourite();
			break;
		case MSG_SIDEBAR_TREEVIEW_CHANGED_DIR:
			ChangedDirectoryTreeView( pcMessage );
			break;

		// Address message
		case MSG_DIRBAR_CHANGE_DIR:
			ChangeDirBarDirectory( pcMessage );
			break;

		// Browser messages
		case MSG_BROWSER_SELECTION_CHANGED:
			SelectionChanged( pcMessage );		
			break;
		case MSG_BROWSER_CHANGE_DIR:
			ChangedDirectory( pcMessage );		
			break;
		case MSG_BROWSER_UPDATE_STATUSBAR:
			UpdateStatusBar();
			break;
		case MSG_BROWSER_MOUNT:
			Mounting( pcMessage );
			break;
		case MSG_BROWSER_UNMOUNT:
			Unmounting( pcMessage );
			break;

		// Misc
		case MSG_FAVOURITES_ADD:
			AddFavourite( pcMessage );
			break;

		default:
		{
			Window::HandleMessage( pcMessage );
			break;
		}
	}
}

bool AppWindow :: OkToQuit( void )
{
	Application::GetInstance()->PostMessage( M_QUIT );
	return true;
}

void AppWindow :: SetSideBar( SideBar* sidebar )
{
	// Uncheck all items
	m_ViewMenu_SideBarTreeView->SetChecked( false );
	m_ViewMenu_SideBarFileInfo->SetChecked( false );
	m_ViewMenu_SideBarAttributes->SetChecked( false );
	m_ViewMenu_SideBarSearch->SetChecked( false );
	m_ViewMenu_SideBarFavourites->SetChecked( false );
	m_ViewMenu_SideBarHide->SetChecked( false );

	// Want to hide sidebar
	if( sidebar == NULL )
	{
		if( m_CurrentSideBar != NULL && m_CurrentSideBar->IsVisible() )
		{
			m_CurrentSideBar->Hide();
			m_CurrentSideBar = NULL;
			Layout();
		}
	}
	else if( m_CurrentSideBar != sidebar ) 
	{
		if( m_CurrentSideBar != NULL )
			m_CurrentSideBar->Hide();	

		m_CurrentSideBar = NULL;

		try
		{
			BrowserTab* tab = GetCurrentBrowserTab();

			m_CurrentSideBar = sidebar;
			m_CurrentSideBar->Show();

			m_CurrentSideBar->SetPath( tab->GetPath() );
			vector<String> ret = tab->GetSelectedFiles();
			m_CurrentSideBar->SelectionChanged( ret );
			

		}
		catch( ... ) {	}

		Layout();
	}
}

void AppWindow :: SelectionChanged( Message* pcMessage )
{
	BrowserTab* tab;

	if( pcMessage->FindPointer( "browsertab", (void**) &tab ) != 0 )
		return;

	assert( tab != NULL );

	vector<String> ret = tab->GetSelectedFiles();

	if( m_CurrentSideBar != NULL )
		m_CurrentSideBar->SelectionChanged( ret );
}

void AppWindow :: ChangedDirectory( Message* pcMessage )
{
	// Extract the path
	String path;

	if( pcMessage->FindString( "file/path", &path.str() ) != 0 )
		return;

	// Update the address bars
	m_DirBar->SetPath( path );
	m_DirTextBar->SetPath( path );
	UpdateStatusBar();
	if( m_CurrentSideBar != NULL )
		m_CurrentSideBar->SetPath( path );
	
}

void AppWindow :: ChangedDirectoryTreeView( Message* pcMessage )
{
	// Extract the path
	String path;

	if( pcMessage->FindString( "file/path", &path.str() ) != 0 )
		return;

	try
	{
		BrowserTab* tab = GetCurrentBrowserTab();
		tab->SetPath( path );	
		m_DirBar->SetPath( path );
		m_DirTextBar->SetPath( path );
		UpdateStatusBar();	
	}
	catch( ... ) {	}
}

void AppWindow :: ToolbarBack()
{
	try
	{
		BrowserTab* tab = GetCurrentBrowserTab();
		tab->Back();
		m_DirBar->SetPath( tab->GetPath() );
		m_DirTextBar->SetPath( tab->GetPath() );
		UpdateStatusBar();
	}
	catch( ... )
	{
	}
}

void AppWindow :: ToolbarUp()
{
	try
	{
		BrowserTab* tab = GetCurrentBrowserTab();
		std::string current_path = tab->GetPath();

		// If we are already at the root then stop
		if( current_path == "/" )
			return;

		int i = current_path.rfind( "/" );

		if( i == std::string::npos )
			return;

		if( i == 0 )
			current_path.resize( 1 );
		else
			current_path.resize( i );

		tab->SetPath( current_path );
		m_DirBar->SetPath( tab->GetPath() );
		m_DirTextBar->SetPath( tab->GetPath() );
		UpdateStatusBar();

	}
	catch( ... )
	{
	}
}

void AppWindow :: ToolbarHome()
{
	try
	{
		BrowserTab* tab = GetCurrentBrowserTab();
		tab->SetPath( getenv( "HOME" ) );
	}
	catch( ... ) {	}
}

void AppWindow :: Search( Message* pcMessage )
{
	String path;
	SearchCriteria searchcriteria;

	try
	{
		BrowserTab* tab = GetCurrentBrowserTab();
		path = tab->GetPath();
	}
	catch( ... ) {	return; }

	String filename_filter;
	if( pcMessage->FindString( "filename_filter", &filename_filter.str() ) != 0 )
		return;

	String content;
	if( pcMessage->FindString( "content", &content.str() ) != 0 )
		return;

	bool recursive;
	if( pcMessage->FindBool( "recursive", &recursive ) != 0 )
		return;

	// Setup criteria
	searchcriteria.SetRecursive( recursive );
	searchcriteria.SetFilenameCriteria( filename_filter );
	searchcriteria.SetContentCriteria( content );

	SearchTab* searchview = new SearchTab( Rect(), "" );	
	m_Tab->SetSelection( m_Tab->AppendTab( "Search results", searchview) );
	searchview->SetSearchCriteria( path, &searchcriteria );
	searchview->DoSearch();
	PopulateTabs();
}

BrowserTab* AppWindow :: GetCurrentBrowserTab()
{
	if( ( m_Tab->GetTabCount() < 1 ) || ( (int) m_Tab->GetSelection() < 0 ) )
		throw String( "No tabs found" );

	BrowserTab* tab = dynamic_cast<BrowserTab*>( m_Tab->GetTabView( m_Tab->GetSelection() ) );

	if( tab == NULL )
		throw String( "No browser tab found" );
	
	return tab;
}

void AppWindow :: ChangeDirBarDirectory( Message* pcMessage )
{
	String path;
	if( pcMessage->FindString( "file/path", &path.str() ) != 0 )
		return;
		
	try
	{
		BrowserTab* tab = GetCurrentBrowserTab();
		tab->SetPath( path );
		UpdateStatusBar();
	}
	catch( ... ) {	}
}

void AppWindow :: AddQuickFavourite()
{
	try
	{
		BrowserTab* tab = GetCurrentBrowserTab();
		if( !BrowserApp::GetFavourites()->NameInUse( tab->GetPath() ) )
			BrowserApp::GetFavourites()->AddFavourite( tab->GetPath(), tab->GetPath() );
		PopulateFavourites();
	}
	catch( ... ) {	}
}

void AppWindow :: AddFavourite( Message* msg )
{
	String name, path;
	if( msg->FindString( "path", &path.str() ) == 0 &&  msg->FindString( "name", &name.str() ) == 0 )
		BrowserApp::GetFavourites()->AddFavourite( name, path );

	PopulateFavourites();
}

void AppWindow :: PopulateFavourites()
{
	// Populate the sidebar
	if( m_SideBarFavourites )
		m_SideBarFavourites->Populate( BrowserApp::GetFavourites() );

	// Poopulate the admin
	if( m_DiaFavouritesAdmin )
		m_DiaFavouritesAdmin->Populate();

	// Remove all items in the Favourite menu
	while( m_FavouritesMenu->GetItemCount() > 2 )
		m_FavouritesMenu->RemoveItem( 2 );

	// Populate the menu
	if( BrowserApp::GetFavourites()->GetFavouritesCount() > 0 )
	{
		m_FavouritesMenu->AddItem( new MenuSeparator() );

		for( int i = 0 ; i < BrowserApp::GetFavourites()->GetFavouritesCount() ; i++ )
		{
			String name, path;

			BrowserApp::GetFavourites()->GetFavourite( i, name, path );

			Message* msg = new Message( MSG_DIRBAR_CHANGE_DIR );
			msg->AddString( "file/path", path );

			String number;			
			number.Format( "%d. ", i + 1 );

			MenuItem* m = new MenuItem( number + name, msg );
			m_FavouritesMenu->AddItem( m );
		}

	}

	m_MenuBar->SetTargetForItems( this );
}

void AppWindow :: ShowFavouritesAddDialog()
{
	if( !m_DiaFavouritesAdd )
	{
		m_DiaFavouritesAdd = new DiaFavouritesAdd( this, Rect( 100,100, 300, 180 ) );
		m_DiaFavouritesAdd->CenterInWindow( this );
		m_DiaFavouritesAdd->Run();
	}

	BrowserTab* tab = NULL;

	try
	{
		tab = GetCurrentBrowserTab();
	}
	catch( ... ) { }

	if( tab != NULL )
		m_DiaFavouritesAdd->Raise( tab->GetPath() );
	else
		m_DiaFavouritesAdd->Raise( "" );
}

void AppWindow :: ShowFavouritesAdminDialog()
{
	if( !m_DiaFavouritesAdmin )
	{
		m_DiaFavouritesAdmin = new DiaFavouritesAdmin( this, BrowserApp::GetFavourites(), Rect( 100,100, 400, 280 ) );
		m_DiaFavouritesAdmin->Populate();
		m_DiaFavouritesAdmin->CenterInWindow( this );
		m_DiaFavouritesAdmin->Run();
	}

	m_DiaFavouritesAdmin->Raise();
}

void AppWindow :: SetView( IconDirectoryView::view_type type )
{
	try
	{
		BrowserTab* tab = GetCurrentBrowserTab();
		tab->SetView( type );
	}
	catch( ... ) { }
}

void AppWindow :: PopulateTabs()
{
	// Remove all items in the Tabs menu
	while( m_TabsMenu->GetItemCount() > 8 )
		m_TabsMenu->RemoveItem( 8 );

	// Populate the menu
	if( m_Tab->GetTabCount() > 0 )
	{
		m_TabsMenu->AddItem( new MenuSeparator() );

		for( int i = 0 ; i < m_Tab->GetTabCount() ; i++ )
		{
			String name = m_Tab->GetTabTitle( i );

			Message* msg = new Message( MSG_MENU_TABS_ACTIVATE_TAB );
			msg->AddInt16( "tab_no", i );

			String number;			
			number.Format( "%d. ", i + 1 );

			CheckMenu* m = new CheckMenu( number + name, msg );
			m_TabsMenu->AddItem( m );
		}

	}

	SetTab( m_Tab->GetSelection(), false );
	m_MenuBar->SetTargetForItems( this );
}

void AppWindow :: CloseTab( int close )
{
	if( m_Tab == NULL )
		return;

	// Make sure there is tabs
	if( m_Tab->GetTabCount() < 1 )
		return;

	switch( close )
	{
		case MSG_MENU_TABS_CLOSE:
			delete m_Tab->DeleteTab( m_Tab->GetSelection() );
			break;	
		case MSG_MENU_TABS_CLOSE_ALL_OTHER:
		{
			int sel = m_Tab->GetSelection();

			for( int i = 0; i < ( sel - 1 ) ; i++ )
				delete m_Tab->DeleteTab( 0 );

			while( m_Tab->GetTabCount() > 1 )
				delete  m_Tab->DeleteTab( 1 );	
			
			m_Tab->SetSelection( 0 );
			break;
		}
		case MSG_MENU_TABS_CLOSE_ALL:
		{
			while( m_Tab->GetTabCount() > 0 )
				delete m_Tab->DeleteTab( 0 );		
			break;
		}
		default:
			break;
	}

	m_Tab->Invalidate();
	m_Tab->Flush();

	PopulateTabs();
}

void AppWindow :: CycleTab( int direction )
{
	// Make sure there is at least tabs
	if( m_Tab->GetTabCount() < 2 )
		return;

	switch( direction )
	{
		case MSG_MENU_TABS_NEXT_TAB:
		{		
			// Get current selected tab
			int sel = m_Tab->GetSelection();
	
			// Calc next selection. If we have reach the end of the tab list we start over
			if( sel < m_Tab->GetTabCount() - 1 )
				sel++;
			else
				sel = 0;

			SetTab( sel, true );
			break;
		}
		case MSG_MENU_TABS_PREVIOUS_TAB:
		{
			// Get current selected tab
			int sel = m_Tab->GetSelection();
	
			// Calc next selection. If we have reach the end of the tab list we start over
			if( sel > 0)
				sel--;
			else
				sel = m_Tab->GetTabCount() - 1;

			SetTab( sel, true );
			break;
		}
		default:
			return;
	}
}

void AppWindow :: SetTab( int tab, bool notify )
{
	m_Tab->SetSelection( tab, notify );

	// Update the tab menu
	for( int i = 9 ; i < m_TabsMenu->GetItemCount() ; i++ )
	{
		CheckMenu* m = dynamic_cast<CheckMenu*>( m_TabsMenu->GetItemAt( i ) );
		if( m )
			m->SetChecked( ( i - 9 ) == tab );
	}

	try
	{
		BrowserTab* tab = GetCurrentBrowserTab();

		// Update the address bars
		m_DirBar->SetPath( tab->GetPath() );
		m_DirTextBar->SetPath( tab->GetPath() );
		UpdateStatusBar();
	}
	catch ( ... ) { } 
}

void AppWindow :: UpdateStatusBar()
{
	try
	{
		BrowserTab* tab = GetCurrentBrowserTab();
		tab->SelectionChanged();
	}
	catch ( ... ) { } 
}

void AppWindow :: Unmounting( Message* msg )
{
	
}

void AppWindow :: Mounting( Message* msg )
{
}

