// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef __APPWINDOW_H_
#define __APPWINDOW_H_

#include <util/message.h>
#include <util/string.h>
#include <util/resources.h>
#include <util/settings.h>

#include <gui/window.h>
#include <util/string.h>
#include <gui/image.h>
#include <gui/menu.h>
#include <gui/checkmenu.h>
#include <gui/filerequester.h>
#include <gui/requesters.h>
#include <gui/toolbar.h>
#include <gui/icondirview.h>

#include "sidebar.h"
#include "sidebar_search.h"
#include "sidebar_favourites.h"
#include "sidebar_attributes.h"
#include "sidebar_treeview.h"
#include "sidebar_fileinfo.h"
#include "dirbar.h"
#include "dirtextbar.h"
#include "browsertab.h"
#include "favourites.h"
#include "dia_fav_add.h"
#include "dia_fav_admin.h"
#include "tabview.h"

class AppWindow : public os::Window
{
public:
	AppWindow( const os::Rect& cFrame );
	~AppWindow();
	virtual void HandleMessage( os::Message* pcMessage );
	virtual bool OkToQuit( void );

private:
	void Layout( bool force = false );
	void SetSideBar( SideBar* sidebar );
	void SelectionChanged( os::Message* pcMessage );
	void ChangedDirectory( os::Message* pcMessage );
	void ToolbarBack();
	void ToolbarHome();
	void ToolbarUp();
	void Search( os::Message* pcMessage );
	BrowserTab* GetCurrentBrowserTab();
	void ChangeDirBarDirectory( os::Message* pcMessage );
	void ChangedDirectoryTreeView( os::Message* pcMessage );
	void AddFavourite( os::Message* pcMessage );
	void AddQuickFavourite();
	void PopulateFavourites();
	void ShowFavouritesAddDialog();
	void ShowFavouritesAdminDialog();
	void SetView( os::IconDirectoryView::view_type type );
	void PopulateTabs();
	void CloseTab( int close );
	void CycleTab( int direction );
	void SetTab( int tab, bool notify = true );
	void UpdateStatusBar();
	void Unmounting( os::Message* msg );
	void Mounting( os::Message* msg );

	// Menu
	os::Menu* m_MenuBar;
	os::Menu* m_ApplicationMenu;
	os::Menu* m_ViewMenu;
	os::Menu* m_FavouritesMenu;
	os::Menu* m_TabsMenu;
	
	os::CheckMenu* m_ViewMenu_Menu;
	os::CheckMenu* m_ViewMenu_ToolBar;
	os::CheckMenu* m_ViewMenu_AddressBar_Text;
	os::CheckMenu* m_ViewMenu_AddressBar_Button;
	os::CheckMenu* m_ViewMenu_AddressBar_Hide;
	os::CheckMenu* m_ViewMenu_SideBarTreeView;
	os::CheckMenu* m_ViewMenu_SideBarFileInfo;
	os::CheckMenu* m_ViewMenu_SideBarAttributes;
	os::CheckMenu* m_ViewMenu_SideBarSearch;
	os::CheckMenu* m_ViewMenu_SideBarFavourites;
	os::CheckMenu* m_ViewMenu_SideBarHide;
	os::CheckMenu* m_ViewMenu_StatusBar;
	os::CheckMenu* m_ViewMenu_TabAutomatic;
	os::CheckMenu* m_ViewMenu_TabShow;
	os::CheckMenu* m_ViewMenu_TabHide;

	os::CheckMenu* m_ViewMenu_BrowserIcons;
	os::CheckMenu* m_ViewMenu_BrowserList;
	os::CheckMenu* m_ViewMenu_BrowserDetails;

	os::Menu* m_SidebarMenu;
	os::Menu* m_AddressBarMenu;
	os::Menu* m_TabSubMenu;

	os::MenuItem* m_FavouritesMenu_Add;
	
	// Toolbar
	os::ToolBar* m_ToolBar;

	// Address toolbar
	DirBar* m_DirBar;
	DirTextBar* m_DirTextBar;

	// The tab area
	TabView* m_Tab;

	// The sidebars
	TreeViewSideBar* m_SideBarTreeView;
	AttributesSideBar* m_SideBarAttributes;
	SearchSideBar* m_SideBarSearch;
	FavouritesSideBar* m_SideBarFavourites;
	FileInfoSideBar* m_SideBarFileInfo;

	SideBar* m_CurrentSideBar;

	// Dialogs
	DiaFavouritesAdd* m_DiaFavouritesAdd;
	DiaFavouritesAdmin* m_DiaFavouritesAdmin;
};

#endif /* __APPWINDOW_H_ */


