// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "appwindow.h"
#include "dia_fav_admin.h"
#include "messages.h"

using namespace os;

DiaFavouritesAdmin :: DiaFavouritesAdmin( Looper* parent, Favourites* favourites, const Rect& cFrame ) : Window( cFrame, "main_window", "Administrate Favourites" )
{	
	m_Parent = parent;
	m_Favourites = favourites;
	m_DiaFavouritesNew = NULL;
	m_DiaFavouritesModify = NULL;

	m_New = new Button( Rect(), "button_new", "New", new Message( MSG_DIA_ADMIN_NEW ) );
	m_New->SetTarget( this );
	AddChild( m_New );

	m_Modify = new Button( Rect(), "button_modify", "Modify", new Message( MSG_DIA_ADMIN_MODIFY ) );
	m_Modify->SetTarget( this );
	AddChild( m_Modify );

	m_Remove = new Button( Rect(), "button_remove", "Remove", new Message( MSG_DIA_ADMIN_REMOVE ) );
	m_Remove->SetTarget( this );
	AddChild( m_Remove );

	m_Links = new ListView( Rect(), "links" );
	m_Links->InsertColumn( "Name", 130 );
	m_Links->InsertColumn( "Path", 180 );
	m_Links->SetMultiSelect( false );
	m_Links->SetAutoSort( false );
	m_Links->SetInvokeMsg( new Message( MSG_DIA_ADMIN_SELECT ) );
	AddChild( m_Links );	

	Layout();
}

DiaFavouritesAdmin :: ~DiaFavouritesAdmin()
{
	delete m_New; 
	delete m_Modify; 
	delete m_Remove; 
	delete m_Links;
	delete m_DiaFavouritesNew;
	delete m_DiaFavouritesModify;
}


void DiaFavouritesAdmin :: FrameSized( const Point& delta )
{
	Window::FrameSized( delta );

	Layout();
}

void DiaFavouritesAdmin :: Layout()
{
	float offset_x = 4.0f;
	float offset_y = 4.0f;
	float button_width;

	Rect cFrame = GetBounds();

	button_width = ( cFrame.right - cFrame.left - offset_x * 2 - offset_x - offset_x ) / 3.0f;

	Point cButtonNew = m_New->GetPreferredSize( false );
	Rect cNewFrame( offset_x, cFrame.bottom - offset_y - cButtonNew.y, offset_x + button_width, cFrame.bottom - offset_y );
	m_New->SetFrame( cNewFrame );

	Point cButtonModify = m_Modify->GetPreferredSize( false );
	Rect cModifyFrame( cNewFrame.right + offset_x, cFrame.bottom - offset_y - cButtonModify.y, cNewFrame.right + offset_x + button_width, cFrame.bottom - offset_y );
	m_Modify->SetFrame( cModifyFrame );

	Point cButtonRemove = m_Remove->GetPreferredSize( false );
	Rect cRemoveFrame( cModifyFrame.right + offset_x, cFrame.bottom - offset_y - cButtonRemove.y, cModifyFrame.right + offset_x + button_width, cFrame.bottom - offset_y );
	m_Remove->SetFrame( cRemoveFrame );

	Rect cLinkFrame( offset_x, cFrame.top + offset_y, cFrame.right - offset_x, cRemoveFrame.top - offset_y );
	m_Links->SetFrame( cLinkFrame );
}

void DiaFavouritesAdmin :: Raise()
{
	if( IsVisible() )
		Hide();

	Show();
	MakeFocus();
}

void DiaFavouritesAdmin :: HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )	//Get the message code from the message
	{
		case MSG_DIA_ADMIN_NEW:
			NewLink();
			break;
		case MSG_DIA_ADMIN_MODIFY:
			ModifyLink();
			break;
		case MSG_DIA_ADMIN_REMOVE:
			DeleteLink();
			break;
		case MSG_DIA_ADMIN_SELECT:
			SelectLink();
			break;

		case MSG_FAVOURITES_ADD:
			m_Parent->PostMessage( pcMessage, m_Parent );
			break;

		default:
			Window::HandleMessage( pcMessage );
			break;
	}
}

bool DiaFavouritesAdmin :: OkToQuit( void )
{
	Hide();
	return false;
}

void DiaFavouritesAdmin :: Populate()
{
	m_Links->Clear();

	for( int i = 0 ; i < m_Favourites->GetFavouritesCount() ; i ++ )
	{
		String name, path;

		m_Favourites->GetFavourite( i, name, path );

		// Add to the listview
		ListViewStringRow* pcRow = new ListViewStringRow();
		pcRow->SetCookie( name );
		pcRow->AppendString( name );
		pcRow->AppendString( path );
		m_Links->InsertRow( pcRow );
	}
}

void DiaFavouritesAdmin :: NewLink()
{
	if( !m_DiaFavouritesNew )
	{
		m_DiaFavouritesNew = new DiaFavouritesAdd( this, Rect( 100,100, 300, 180 ), String( "New Favourite" ) );
		m_DiaFavouritesNew->CenterInWindow( this );
		m_DiaFavouritesNew->Run();
	}

	m_DiaFavouritesNew->Lock();
	m_DiaFavouritesNew->Raise( "" );
	m_DiaFavouritesNew->Unlock();
}

void DiaFavouritesAdmin :: ModifyLink()
{
	if( !m_DiaFavouritesModify )
	{
		m_DiaFavouritesModify = new DiaFavouritesModify( this, Rect( 100,100, 300, 180 ) );
		m_DiaFavouritesModify->CenterInWindow( this );
		m_DiaFavouritesModify->Run();
	}

	ListViewRow* row = m_Links->GetRow( m_Links->GetLastSelected() );

	m_DiaFavouritesModify->Lock();
	if( row != NULL )
		m_DiaFavouritesModify->Raise( row->GetCookie().AsString() );
	else
		m_DiaFavouritesModify->Raise( "" );
	m_DiaFavouritesModify->Unlock();
}

void DiaFavouritesAdmin :: DeleteLink()
{
	ListViewRow* row = m_Links->GetRow( m_Links->GetLastSelected() );

	if( row != NULL )
		m_Favourites->DeleteFavourite( row->GetCookie().AsString() );

	// Force a populate update
	Message msg( MSG_FAVOURITES_ADD );
	m_Parent->PostMessage( &msg, m_Parent );
}

void DiaFavouritesAdmin :: SelectLink()
{
	ListViewRow* row = m_Links->GetRow( m_Links->GetLastSelected() );
	String name = row->GetCookie().AsString();

	Message msg( MSG_DIRBAR_CHANGE_DIR );
	msg.AddString( "file/path", m_Favourites->GetPathFromName( name )  );
	m_Parent->PostMessage( &msg, m_Parent );
}
