// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <gui/requesters.h>
#include <util/string.h>

#include "main.h"
#include "messages.h"
#include "dia_fav_modify.h"

using namespace os;

DiaFavouritesModify :: DiaFavouritesModify( Looper* parent, const Rect& cFrame ) : Window( cFrame, "main_window", "Modify a Favourite" )
{
	m_Parent = parent;

	m_LabelPath = new StringView( Rect(), "label_path", "Path:" );
	AddChild( m_LabelPath );

	m_LabelName = new StringView( Rect(), "label_name", "Name:" );
	AddChild( m_LabelName );

	m_Path = new SLTextView( Rect(), "path", NULL, true );
	AddChild( m_Path );

	m_Name = new SLTextView( Rect(), "name", NULL, false );
	AddChild( m_Name );

	m_Modify = new Button( Rect(), "button_add", "Modify", new Message( MSG_DIA_FAV_MODIFY ) );
	m_Modify->SetTarget( this );
	AddChild( m_Modify );

	m_Cancel = new Button( Rect(), "button_cancel", "Cancel", new Message( MSG_DIA_FAV_CANCEL ) );
	m_Cancel->SetTarget( this );
	AddChild( m_Cancel );

	Layout();
}

DiaFavouritesModify :: ~DiaFavouritesModify()
{
	delete m_LabelPath;
	delete m_LabelName;
	delete m_Path;
	delete m_Name;
	delete m_Modify; 
	delete m_Cancel; 
}

void DiaFavouritesModify :: FrameSized( const Point& delta )
{
	Window::FrameSized( delta );

	Layout();
}

void DiaFavouritesModify :: Layout()
{
	float offset_x = 4.0f;
	float offset_y = 4.0f;

	Rect cFrame = GetBounds();

	Point cLabelPath = m_LabelPath->GetPreferredSize( false );
	Rect cLPathFrame( offset_x, offset_y, offset_x + cLabelPath.x, offset_y + cLabelPath.y );
	m_LabelPath->SetFrame( cLPathFrame );

	Point cEntryPath = m_Path->GetPreferredSize( false );
	Rect cPathFrame( cLPathFrame.right + 4.0f, offset_y, cFrame.right - offset_x, offset_y + cEntryPath.y );
	m_Path->SetFrame( cPathFrame );

	Point cLabelName = m_LabelName->GetPreferredSize( false );
	Rect cLNameFrame( offset_x, cPathFrame.bottom + 4.0f, offset_x + cLabelName.x, cPathFrame.bottom + 4.0f + cLabelName.y );
	m_LabelName->SetFrame( cLNameFrame );

	Point cEntryName = m_Name->GetPreferredSize( false );
	Rect cNameFrame( cLNameFrame.right + 4.0f, cPathFrame.bottom + 4.0f, cFrame.right - offset_x,  cPathFrame.bottom + 4.0f + cEntryPath.y );
	m_Name->SetFrame( cNameFrame );

	Point cCancel = m_Cancel->GetPreferredSize( false );
	Rect cCancelFrame( cFrame.right - offset_x - cCancel.x, cFrame.bottom - offset_y - cCancel.y, cFrame.right - offset_x, cFrame.bottom - offset_y);
	m_Cancel->SetFrame( cCancelFrame );

	Point cAdd = m_Modify->GetPreferredSize( false );
	Rect cAddFrame( cCancelFrame.left - 4.0f - cAdd.x, cFrame.bottom - offset_y - cAdd.y, cCancelFrame.left - 4.0f,  cFrame.bottom - offset_y );
	m_Modify->SetFrame( cAddFrame );
}

void DiaFavouritesModify :: Raise( String old_name )
{
	if( IsVisible() )
		Hide();

	Show();
	MakeFocus();
	m_OldName = old_name;

	// Get old favourite info
	m_Name->SetText( old_name );
	m_Path->SetText( BrowserApp::GetFavourites()->GetPathFromName( old_name ) );
	m_Name->MakeFocus();
}

void DiaFavouritesModify :: HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )	//Get the message code from the message
	{
		case MSG_DIA_FAV_MODIFY:
			ModifyLink();
			break;
		case MSG_DIA_FAV_CANCEL:
			Hide();
			break;
		default:
			Window::HandleMessage( pcMessage );
			break;
	}
}

bool DiaFavouritesModify :: OkToQuit( void )
{
	Hide();
	return false;
}

void DiaFavouritesModify :: ModifyLink()
{
	// Make sure there is a name
	String name = m_Name->GetText();
	name.Strip();

	if( name == "" )
	{
		Alert* alert=new Alert( "Error", "A favourite must have a name", Alert::ALERT_WARNING, "OK", NULL);
		alert->CenterInWindow( this );
		alert->Go(new Invoker());
		return;
	}

	// Make sure the name is not used
	if( BrowserApp::GetFavourites()->NameInUse( name ) )
	{
		Alert* alert=new Alert( "Error", "The name is already in use", Alert::ALERT_WARNING, "OK", NULL);
		alert->CenterInWindow( this );
		alert->Go(new Invoker());
		return;
	}

	// Make sure we have a path
	String path = m_Path->GetText();
	path.Strip();

	if( path == "" )
	{
		Alert* alert=new Alert( "Error", "A favourite must have a path", Alert::ALERT_WARNING, "OK", NULL);
		alert->CenterInWindow( this );
		alert->Go(new Invoker());
		return;
	}

	// No errors found, lets update our favourite list
	try
	{
		BrowserApp::GetFavourites()->UpdateFavourite( m_OldName, m_Name->GetText(), m_Path->GetText() );
		Message msg( MSG_FAVOURITES_ADD );
		m_Parent->PostMessage( &msg, m_Parent );
	}
	catch( ... )
	{
		Alert* alert=new Alert( "Error", "Unable to update favourite", Alert::ALERT_WARNING, "OK", NULL);
		alert->CenterInWindow( this );
		alert->Go(new Invoker());
	}

	Hide();
}
