// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <gui/window.h>
#include "dirtextbar.h"

using namespace os;

DirTextBar::DirTextBar( Rect cFrame, const String& cName, Message* pcMessage, uint32 nResizeMask, uint32 nFlags ) : View(cFrame, cName, nResizeMask, nFlags)
{
	m_theMessage=pcMessage;

	// Create widgets
	m_Text = new StringView( Rect(), "label", "Address:" );
	AddChild( m_Text );

	m_Entry = new SLTextView( Rect(), "entry", m_theMessage, true );	
	AddChild( m_Entry );

	m_Go = new Button( Rect(), "go", "Go", m_theMessage );
	AddChild( m_Go );

	Layout();
}

DirTextBar::~DirTextBar()
{
/*	delete m_theMessage;
	delete m_Text;
	delete m_Entry;
	delete m_Go; */
}

void DirTextBar::AllAttached( void )
{
	View::AllAttached();

	Layout();
	m_Go->SetTarget( this );
}

void DirTextBar::HandleMessage( Message* pcMessage )
{
	if( m_theMessage != NULL )
	{
		m_theMessage->RemoveName("file/path" );
		m_theMessage->AddString( "file/path", GetPath() );
		GetWindow()->HandleMessage( m_theMessage );
	}
}


void DirTextBar::SetFrame( const os::Rect& cRect, bool bNotifyServer )
{
	View::SetFrame( cRect, bNotifyServer );

	Layout();
}

void DirTextBar::Layout()
{
	Rect cRect = GetBounds();

	Point cLabelFrame = m_Text->GetPreferredSize( false );
	Rect cLabel = Rect( 3.0f, 3.0f + 4.0f, cLabelFrame.x + 3.0f, cLabelFrame.y + 3.0f + 4.0f );
	m_Text->SetFrame( cLabel );

	Point cButtonFrame = m_Go->GetPreferredSize( false );
	Rect cButton = Rect( cRect.right - 3.0f - cButtonFrame.x, 3.0f, cRect.right - 3.0f, 3.0f + cButtonFrame.y );
	m_Go->SetFrame( cButton );	

	Point cEntryFrame = m_Entry->GetPreferredSize( false );
	Rect cEntry = Rect( cLabel.right + 3.0f, 3.0f + 2.0f, cButton.left - 3.0f, 3.0f + cEntryFrame.y + 2.0f );
	m_Entry->SetFrame( cEntry );
}


void DirTextBar::SetPath( String path )
{
	// Sanity check
	if(path[0]!='/')
		throw String("Not a valid path");

	// Set new path
	m_Entry->SetValue( path );
}

String DirTextBar::GetPath()
{
	return m_Entry->GetValue();
}

Point DirTextBar::GetPreferredSize(bool bLargest)
{
	float height = m_Text->GetPreferredSize( bLargest ).y;
	if( height < m_Entry->GetPreferredSize( bLargest ).y )
		height = m_Entry->GetPreferredSize( bLargest ).y;
	if( height < m_Go->GetPreferredSize( bLargest ).y )
		height = m_Go->GetPreferredSize( bLargest ).y;

	height += 6.0f; // Plus the borders;

	return Point( 160000.0f, height);
}
