// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <gui/window.h>
#include <gui/menu.h>
#include <storage/fsnode.h>

#include "eicondirview.h"
#include "script.h"
#include "messages.h"

using namespace os;
using namespace std;

EIconDirectoryView :: EIconDirectoryView( const Rect& cFrame, const String& cPath, uint32 nResizeMask ) : IconDirectoryView( cFrame, cPath, nResizeMask )
{
	m_StatusBarChangeMsg = NULL;
}

EIconDirectoryView :: ~EIconDirectoryView()
{
	delete m_StatusBarChangeMsg;
}

void EIconDirectoryView :: AllAttached()
{
	IconDirectoryView::AllAttached();
	SendStatusBarChange();
}

void EIconDirectoryView :: ReRead()
{
	IconDirectoryView::ReRead();
}

void EIconDirectoryView :: HandleMessage( Message* pcMessage )
{
	// Check if there is any particular message
	switch( pcMessage->GetCode() )
	{
		case IconDirectoryView::M_READY:
		{
			SendStatusBarChange();
			break;
		}
		case MSG_BROWSER_CALL_SCRIPT:
		{
		/*	Script* script;
			String function;

			if( pcMessage->FindString( "function", &function.str() ) == 0 &&
				pcMessage->FindPointer( "script", (void **) &script ) == 0)
				script->RunFunction( function ); */
			break;
		}
		default:
			IconDirectoryView::HandleMessage( pcMessage );
	}

}

void EIconDirectoryView :: SetStatusBarChangeMsg( Message* pcMsg )
{
	delete m_StatusBarChangeMsg;
	m_StatusBarChangeMsg = pcMsg;
}

void EIconDirectoryView :: SendStatusBarChange()
{
	if( m_StatusBarChangeMsg == NULL )
		return;

	if( GetWindow() )
		GetWindow()->HandleMessage( m_StatusBarChangeMsg );
}

void EIconDirectoryView :: OpenContextMenu( Point cPosition, bool bMouseOverIcon )
{
	vector<String> l = GetSelectedFiles();
	Menu* menu = new Menu( os::Rect(), "popmenu", ITEMS_IN_COLUMN );

	bool root = ( GetPath() == "/" );
	if( root )
	{
		if( l.size() >= 1 )
			menu->AddItem( "Unmount", new Message( MSG_BROWSER_UNMOUNT ) );		
		else
			menu->AddItem( "Mount...", new Message( MSG_BROWSER_MOUNT ) );
	}
	else
	{
		menu->AddItem( "Open", new Message( MSG_BROWSER_OPEN ) );
		menu->GetItemAt( 0 )->SetEnable( l.size() == 1 );

		if( l.size() > 0 && IsDirectory( l[0] ) )
			menu->AddItem( "Open in new tab", new Message( MSG_BROWSER_OPEN_NEW_TAB ) );
		else 
			menu->AddItem( "Open with...", new Message( MSG_BROWSER_OPEN_WITH ) );
		menu->GetItemAt( 1 )->SetEnable( l.size() == 1 );
	}
	menu->AddItem( new MenuSeparator() );

//	Script::ScriptEventPopupMenu( this, menu );

	// Check if there has been any items added by script
	if( menu->GetItemCount() > 3 )
		menu->AddItem( new MenuSeparator() );

	menu->AddItem( "Copy", new Message( MSG_BROWSER_COPY ) );
	menu->AddItem( "Cut", new Message( MSG_BROWSER_CUT ) );
	menu->AddItem( "Paste", new Message( MSG_BROWSER_PASTE ) );
	menu->AddItem( "Delete", new Message( MSG_BROWSER_DELETE ) );
	menu->AddItem( new MenuSeparator() );
	menu->AddItem( "Rename...", new Message( MSG_BROWSER_RENAME ) );
	menu->AddItem( new MenuSeparator() );
	menu->AddItem( "Properties...", new Message( MSG_BROWSER_PROPERTIES ) );
	menu->SetTargetForItems( GetParent() );

	menu->Open( ConvertToScreen( cPosition ) );

	// Clean up
//	delete menu;
}

vector<String> EIconDirectoryView :: GetSelectedFiles()
{
	vector<String> list_of_selected_files;

	// Get number of selected icons and calculate some statistics for the directory
	for( uint i = 0 ; i < GetIconCount() ; i ++ )
	{	
		if( GetIconSelected( i ) )
		{
			DirectoryIconData* data = static_cast< DirectoryIconData* >( GetIconData( i ) );
			list_of_selected_files.push_back( String( GetPath() + "/" + data->m_zPath ) );	
		}
	}

	return list_of_selected_files;
}

bool EIconDirectoryView :: IsDirectory( String path)
{
	try
	{
		FSNode fs( path );
		return ( fs.IsValid() && fs.IsDir() );
	}
	catch( ... ) { };

	return false;
}
