// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include <fnmatch.h>
#include <unistd.h>

#include <storage/directory.h>
#include <storage/path.h>

#include "script.h"
#include "main.h"
#include "messages.h"

/*
extern "C"
{
#include "lualib.h"
#include "lauxlib.h"
}

using namespace os;
using namespace std;

#define SCRIPT_MAGIC_NUMBER 0x12434321

vector< Script* > Script :: m_ListOfAllScripts;
EIconDirectoryView* Script :: m_CurrentIconDirView = NULL;
Menu* Script :: m_CurrentPopupMenu = NULL;

Script :: Script( String path )
{
	printf( "Script: %s\n", path.c_str() );

	// Create new lua state
	L = lua_open();

	// Open lua libs
	luaopen_base( L );
	luaopen_table( L );
	luaopen_io( L );
	luaopen_string( L );
	luaopen_math( L );

	// Add functions
	lua_pushcfunction( L, lua_register_event );
	lua_setglobal( L, "register_event" );
	lua_pushcfunction( L, lua_create_menuitem );
	lua_setglobal( L, "create_menuitem" );
	lua_pushcfunction( L, lua_get_files );
	lua_setglobal( L, "get_files" );
	lua_pushcfunction( L, lua_get_selected_files );
	lua_setglobal( L, "get_selected_files" );
	lua_pushcfunction( L, lua_is_directory );
	lua_setglobal( L, "is_directory" );
	lua_pushcfunction( L, lua_is_file );
	lua_setglobal( L, "is_file" );
	lua_pushcfunction( L, lua_extract_filename );
	lua_setglobal( L, "extract_filename" );
	lua_pushcfunction( L, lua_extract_filename_extension );
	lua_setglobal( L, "extract_filename_extension" );
	lua_pushcfunction( L, lua_get_no_files );
	lua_setglobal( L, "get_no_files" );
	lua_pushcfunction( L, lua_get_no_selected_files );
	lua_setglobal( L, "get_no_selected_files" );
	lua_pushcfunction( L, lua_launch );
	lua_setglobal( L, "launch" );
	lua_pushcfunction( L, lua_set_path );
	lua_setglobal( L, "set_path" );
	lua_pushcfunction( L, lua_get_path );
	lua_setglobal( L, "get_path" );

	// Register the script
	lua_pushlightuserdata( L, (void *) SCRIPT_MAGIC_NUMBER );
	lua_pushlightuserdata( L, (void *) this );
	lua_settable( L, LUA_REGISTRYINDEX );

	// load script
	if( luaL_loadfile( L, path.c_str() ) )
		throw( String( lua_tostring( L, -1 ) ) );

	// run the init part of the script
	if( lua_pcall( L, 0, 0, 0 ) )
		throw( String( lua_tostring( L, -1 ) ) );
}

Script :: ~Script()
{
	lua_close( L );
}

void Script :: RegisterEvent( String event, String function, String file_mask )
{
	Event ev( function, file_mask );
	m_ListOfEvents[ event ] = ev;
}

void Script :: TriggerEvent( String event )
{
	Event ev = m_ListOfEvents[ event ];

	vector<String> file = m_CurrentIconDirView->GetSelectedFiles();
	vector<String>::iterator it;

	for( it = file.begin() ; it != file.end(); it++ )
	{
		if( ev.Trigger( *it ) )
		{
			ev.Run( L );
			return;
		}
	}
}


//////////////////////////////
// STATIC FUNCTIONS
//////////////////////////////

void Script :: LoadAllScripts()
{
	String cName;
	String path = BrowserApp::GetSettings()->GetScriptPath();

	try
	{
		Directory dir( path );
		dir.Rewind();

		while( dir.GetNextEntry( &cName ) == 1 )
		{
			if( !( cName == "." ) && !( cName == ".." ) )
			{
				String new_path, path;
				dir.GetPath( &path );
				new_path = path + String( "/" ) + cName;
				try
				{
					m_ListOfAllScripts.push_back( new Script( new_path ) );			
				} 
				catch( String e )
				{
					printf( "Error: %s\n", e.c_str() );
				}
				catch( ... ) { }
			}
		}
	}
	catch( ... ) { }
}

void Script :: ScriptEventPopupMenu( EIconDirectoryView* browser, Menu* menu )
{
	vector< Script* >::iterator it;

	m_CurrentIconDirView = browser;
	m_CurrentPopupMenu = menu;

	for( it = m_ListOfAllScripts.begin() ; it != m_ListOfAllScripts.end() ; it++ )
	{
		(*it)->TriggerEvent( "popup_menu" );
	}
}

void Script :: RunFunction( String function )
{
	lua_getglobal( L, function.c_str() );
	if( lua_pcall( L, 0, 0, 0 ) )
		throw( String( lua_tostring( L, -1 ) ) );
}

////////////////////////////
// LUA FUNCTIONS
////////////////////////////

int Script :: lua_register_event( lua_State* L )
{
	const char* event;
	const char* function;
	const char* file_mask;
	const char* file_mask_default = "*";

	// Extract parameters
	switch( lua_gettop( L ) )
	{
	case 2:
		event = luaL_checkstring( L, 1 );
		function = luaL_checkstring( L, 2 );
		file_mask = file_mask_default;
		break;
	case 3:
		event = luaL_checkstring( L, 1 );
		function = luaL_checkstring( L, 2 );
		file_mask = luaL_checkstring( L, 3 );
		break;
	default:
		break;
	}

	// Extract the script and call it
	lua_pushlightuserdata( L, (void *) SCRIPT_MAGIC_NUMBER );
	lua_gettable( L, LUA_REGISTRYINDEX );
	Script* script = (Script *) lua_touserdata( L, -1 );
	script->RegisterEvent( String( event ), String( function ), String( file_mask ) );

	return 0;
}

int Script :: lua_create_menuitem( lua_State* L )
{
	const char* item;
	const char* function;

	luaL_checktype(L, 1, LUA_TSTRING);

	item = luaL_checkstring( L, 1 );
	function = luaL_checkstring( L, 2 );

	// Extract the script
	lua_pushlightuserdata( L, (void *) SCRIPT_MAGIC_NUMBER );
	lua_gettable( L, LUA_REGISTRYINDEX );
	Script* script = (Script *) lua_touserdata( L, -1 );

	// Create the menuitem and add it to the menu
	if( m_CurrentPopupMenu != NULL )
	{
		Message* msg = new Message( MSG_BROWSER_CALL_SCRIPT );
	
		msg->AddString( "function", function );
		msg->AddPointer( "script", (void *) script );

		m_CurrentPopupMenu->AddItem( item, msg );
	}

	return 0;
}

int Script :: lua_get_files( lua_State* L )
{
	if( m_CurrentIconDirView == NULL )
	{
		lua_pushnil( L );
		lua_pushstring( L, "Script::lua_get_files: m_CurrentIconDirView == NULL" );
		return 2;
	}

	lua_newtable( L );
	
	for( uint i = 0 ; i < m_CurrentIconDirView->GetIconCount(); i++ )
	{
		Path cPath = m_CurrentIconDirView->GetPath();

		lua_pushnumber( L, i + 1 );		// In Lua tables normally starts with index 1

		// Create filename with path		
		DirectoryIconData *pcData = static_cast < DirectoryIconData * >( m_CurrentIconDirView->GetIconData( i ) );
		cPath.Append( pcData->m_zPath );
		lua_pushstring( L, cPath.GetPath().c_str() );
		lua_settable( L, -3 );
	}

	return 1;
}

int Script :: lua_get_selected_files( lua_State* L )
{
	if( m_CurrentIconDirView == NULL )
	{
		lua_pushnil( L );
		lua_pushstring( L, "Script::lua_get_files: m_CurrentIconDirView == NULL" );
		return 2;
	}

	lua_newtable( L );

	for( uint i = 0, c = 1; i < m_CurrentIconDirView->GetIconCount(); i++ )
	{
		if( m_CurrentIconDirView->GetIconSelected( i ) )
		{
			Path cPath = m_CurrentIconDirView->GetPath();

			lua_pushnumber( L, c );		// In Lua tables normally starts with index 1

			// Create filename with path		
			DirectoryIconData *pcData = static_cast < DirectoryIconData * >( m_CurrentIconDirView->GetIconData( i ) );
			cPath.Append( pcData->m_zPath );
			lua_pushstring( L, cPath.GetPath().c_str() );
			lua_settable( L, -3 );

			c++;
		}
	}

	return 1;
}

int Script :: lua_is_directory( lua_State* L )
{
	const char* path = luaL_checkstring( L, 1 );

	try
	{
		FSNode fs( path );

		lua_pushboolean( L, ( fs.IsValid() && fs.IsDir() ) );
		return 1;
	}
	catch( ... ) { };

	lua_pushnil( L );
	lua_pushstring( L, "Script::is_directory: Unable to open path" );
	
	return 2;
}

int Script :: lua_is_file( lua_State* L )
{
	const char* path = luaL_checkstring( L, 1 );

	try
	{
		FSNode fs( path );

		lua_pushboolean( L, ( fs.IsValid() && fs.IsFile() ) );
		return 1;
	}
	catch( ... ) { };

	lua_pushnil( L );
	lua_pushstring( L, "Script::is_directory: Unable to open path" );
	
	return 2;
}

int Script :: lua_extract_filename( lua_State* L )
{
	const char* path = luaL_checkstring( L, 1 );
	Path p( path );
	lua_pushstring( L, p.GetLeaf().c_str() );
	return 1;
}

int Script :: lua_extract_filename_extension( lua_State* L )
{
	string path = luaL_checkstring( L, 1 );
	string extension;

	int pos = path.rfind( "." );

	if( pos != string::npos )
		extension = path.substr( 	pos + 1, path.size() );

	lua_pushstring( L, extension.c_str() );
	return 1;
}

int Script :: lua_get_no_files( lua_State* L )
{
	if( m_CurrentIconDirView == NULL )
	{
		lua_pushnil( L );
		lua_pushstring( L, "Script::lua_get_no_files: m_CurrentIconDirView == NULL" );
		return 2;
	}

	lua_pushnumber( L, m_CurrentIconDirView->GetIconCount() );
	return 1;
}

int Script :: lua_get_no_selected_files( lua_State* L )
{
	if( m_CurrentIconDirView == NULL )
	{
		lua_pushnil( L );
		lua_pushstring( L, "Script::lua_get_no_selected_files: m_CurrentIconDirView == NULL" );
		return 2;
	}

	int c = 0;
	for( uint i = 0; i < m_CurrentIconDirView->GetIconCount(); i++ )
	{
		if( m_CurrentIconDirView->GetIconSelected( i ) )
			c++;
	}
	
	lua_pushnumber( L, c );
	return 1;
}

int Script :: lua_launch( lua_State* L )
{
	// Should be enough parameters to an app?
	char* p[1024];

	// Extract parameters
	switch( lua_gettop( L ) )
	{
	case 1:
		p[ 0 ] = (char*) luaL_checkstring( L, 1 );
		p[ 1 ] = NULL;
		break;
	case 2:
	{
		int i, n;

		p[ 0 ] = (char*) luaL_checkstring( L, 1 );

		luaL_checktype( L, 2, LUA_TTABLE );
	
		n  = luaL_getn( L, 2 );

		for( i = 1 ; i <= n ; i++ )
		{
			lua_rawgeti( L, 2, i );
			p[ i ] = (char*) luaL_checkstring( L, i + 2 );
		}

		p[ i ] = NULL;

		break;
	}
	default:
		return 0;
	}
	

	if( fork() == 0 )
	{
		set_thread_priority( -1, 0 );
		execvp( p[0], p );
	}
  
	return 0;
}

int Script :: lua_set_path( lua_State* L )
{
	if( m_CurrentIconDirView == NULL )
	{
		lua_pushnil( L );
		lua_pushstring( L, "Script::lua_set_path: m_CurrentIconDirView == NULL" );
		return 2;
	}

	const char* path = luaL_checkstring( L, 1 );

	m_CurrentIconDirView->SetPath( Path( path ).GetPath() );
	m_CurrentIconDirView->ReRead();

	return 0;
}

int Script :: lua_get_path( lua_State* L )
{
	if( m_CurrentIconDirView == NULL )
	{
		lua_pushnil( L );
		lua_pushstring( L, "Script::lua_get_path: m_CurrentIconDirView == NULL" );
		return 2;
	}

	String path = m_CurrentIconDirView->GetPath();

	lua_pushstring( L, path.c_str() );

	return 1;
}

////////////////////////////////////////////////
// EVENT
////////////////////////////////////////////////

Event :: Event()
{
}

Event :: Event( String function, string filemask )
{

	m_Function = function;

	// Extract fileextensions
	int stop_pos = 0;
	int start_pos = 0;
		
	while( 1 )
	{
		stop_pos = filemask.find( ";", start_pos );

		// Add mask
		String s( filemask.substr( start_pos, stop_pos - start_pos ).c_str() );
		s.Strip();
		if( s != "" )
			m_FileMask.push_back( s );

		if( stop_pos == string::npos )
			break;

		start_pos = stop_pos + 1 ;	
	}
}

Event :: ~Event()
{
}

bool Event :: Trigger( String filename )
{
	vector< String >::iterator it;
	Path p( filename );

	for( it = m_FileMask.begin() ; it != m_FileMask.end() ; it++ )
	{
		if( fnmatch( (*it).c_str(),  p.GetLeaf().c_str(),
					 FNM_FILE_NAME | FNM_LEADING_DIR | FNM_PERIOD | FNM_CASEFOLD ) !=  FNM_NOMATCH )
			return true;
	}

	return false;
}

void Event :: Run( lua_State* L )
{
	lua_getglobal( L, m_Function.c_str() );
	if( lua_pcall( L, 0, 0, 0 ) )
		throw( String( lua_tostring( L, -1 ) ) );
}
*/
