// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <string>
#include <fnmatch.h>
#include <storage/file.h>

#include "searchcriteria.h"


using namespace os;
using namespace std;

SearchCriteria :: SearchCriteria()
{
	m_Recursive = true;

	m_CheckFilename = false;
	m_Filename = "";
	m_FilenameCaseSensitive = false;

	m_CheckContent = false;
	m_Content = "";
	m_ContentCaseSensitive = false;
   	m_ContentExactMatch = false;

	m_CheckSize = false;
	m_SizeFunction = LESS;
	m_Size1 = (off_t) 0;
	m_Size2 = (off_t) 0;
} 

SearchCriteria :: ~SearchCriteria()
{
}

void SearchCriteria :: SetFilenameCriteria( os::String filename_filter, bool case_sensitive )
{
	m_Filename = filename_filter;
	m_FilenameCaseSensitive = case_sensitive;
	m_CheckFilename = true;
}

void SearchCriteria :: SetContentCriteria( os::String content, bool case_sensitive, bool exact_match )
{
	m_Content = content;
	m_ContentCaseSensitive = case_sensitive;
	m_ContentExactMatch = exact_match;
	m_CheckContent = ( content != "" );
}

void SearchCriteria :: SetSizeCriteria( enum compare_function type, off_t size_1, off_t size_2 )
{
	m_SizeFunction = type;
	m_Size1 = size_1;
	m_Size2 = size_2;
	m_CheckSize = true;
}

bool SearchCriteria :: FulfillCriteria( os::String filename )
{
	try
	{
		FSNode fs( filename );	

		if( m_CheckFilename )
		{
			if( !CheckFilename( filename ) )
				return false;
		}
	
		if( m_CheckContent )
		{
			if( !CheckContent( fs ) )
				return false;
		}

		if( m_CheckSize )
		{
			if( !CheckSize( fs ) )
				return false;
		}

		return true;
	}
	catch( ... ) { }

	return false;
}

bool SearchCriteria :: CheckFilename( String filename )
{
	string match = string( filename.c_str() );

	// Remove the path from the filename
	uint t = match.rfind( "/" );
	if( t != string::npos )
		match = match.substr( t + 1 );

	int flags = FNM_FILE_NAME | FNM_LEADING_DIR | FNM_PERIOD;

	if( !m_ContentCaseSensitive )
		flags |= FNM_CASEFOLD;

	return fnmatch( m_Filename.c_str(), match.c_str(), flags ) !=  FNM_NOMATCH;
}

bool SearchCriteria :: CheckContent( FSNode fs )
{
	try
	{
		// Create buffer that fits the file
		char* buffer = new char[ fs.GetSize() + 1 ]; 

		// Load the file into the buffer
		File f( fs );

		if( f.Read( buffer, fs.GetSize() ) )
		{
			buffer[ fs.GetSize() ] = '\0';

			string b = buffer;
			string c = m_Content.c_str();

			if( b.find( c ) != string::npos )
			{
				return true;
			}
		}

		delete buffer;
	} 
	catch( ... ) { };

	return false;
}

bool SearchCriteria :: CheckSize( FSNode fs )
{
	off_t size = fs.GetSize();

	switch( m_SizeFunction )
	{
	case LESS:
		return ( size < m_Size1 );
	case MORE:
		return ( size > m_Size1 );
	case BETWEEN:
		return ( size > m_Size1 ) && ( size < m_Size2 );
	}

	return false;
}

