// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef __SEARCHCRITERIA_H_
#define __SEARCHCRITERIA_H_

#include <util/string.h>
#include <storage/fsnode.h>

class SearchCriteria 
{
public:
	enum compare_function { LESS, MORE, BETWEEN };

	SearchCriteria();
	~SearchCriteria();

	void SetRecursive( bool recurv = true ) { m_Recursive = recurv; };
	bool GetRecursive() { return m_Recursive; };

	void SetFilenameCriteria( os::String filename_filter, bool case_sensitive = false );
	void SetContentCriteria( os::String content, bool case_sensitive = false, bool exact_match = false );

	void SetSizeCriteria( enum compare_function type, off_t size_1, off_t size_2 = 0 );

	bool FulfillCriteria( os::String filename );

private:
	bool CheckFilename( os::String filename );
	bool CheckContent( os::FSNode fs );
	bool CheckSize( os::FSNode fs );

	bool m_Recursive;

	// Filename
	bool m_CheckFilename;
	os::String m_Filename;
	bool m_FilenameCaseSensitive;

	// Content
	bool m_CheckContent;
	os::String m_Content;
	bool m_ContentCaseSensitive;
	bool m_ContentExactMatch;

	// Size
	bool m_CheckSize;
	enum compare_function m_SizeFunction;
	off_t m_Size1, m_Size2;
};

#endif

