// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef __SEARCHKEEPER_H_
#define __SEARCHKEEPER_H_

#include <vector>
#include <stack>
#include <util/looper.h>
#include <util/message.h>
#include <util/messenger.h>
#include <util/string.h>
#include <storage/directory.h>
#include <storage/fsnode.h>

#include "searchcriteria.h"

class SearchKeeper : public os::Looper
{
public:	
	SearchKeeper( const os::Messenger& cTarget );
	virtual void HandleMessage( os::Message* pcMessage );
	virtual bool Idle();
	virtual bool OkToQuit();

	os::Directory* GetCurrentDir() { return( m_pcCurrentDir ); }

private:
	void SendSearchResultMessage( const os::String& cName );
	void SendSearchReadyMessage();
	void SendCurrentSearchPath();
	void StartSearch( os::Message* pcMessage );

	enum state_t { S_IDLE, S_READDIR, S_READY };

	SearchCriteria* m_SearchCriteria;

	os::Messenger m_cTarget;
	os::String m_cPath;
	os::Directory* m_pcCurrentDir;
	state_t m_eState;

	bool m_bWaitForAddReply;
	bool m_bWaitForPathReply;
	
	struct FileNode
	{
		FileNode( const os::String& cName, const os::String& cPath )
		{
			m_cName = cName;
			m_cPath = cPath;
		}

		mutable os::String m_cName;
		mutable os::String m_cPath;	
	};

	std::vector < FileNode > m_cAddMap;
	std::vector < os::String > m_cPathMap;
	std::stack < os::Directory* > m_cDirectoryStack;

	std::vector < os::String > m_cHistory;
};

#endif
