// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <util/message.h>
#include <gui/window.h>
#include <storage/fsnode.h>
#include <atheos/fs_attribs.h>

#include "sidebar_attributes.h"
#include "messages.h"

using namespace os;
using namespace std;

AttributesSideBar :: AttributesSideBar( const Rect& cFrame, const String& cTitle, uint32 nResizeMask, uint32 nFlags ) : SideBar( cFrame, cTitle, nResizeMask, nFlags )
{
	m_Links = new ListView( Rect(), "attributes" );
	m_Links->InsertColumn( "Key", 90 );
	m_Links->InsertColumn( "Value", 90 );
	m_Links->SetMultiSelect( false );
	m_Links->SetAutoSort( false );
	AddChild( m_Links );	

	m_Show = new CheckBox( Rect(), "show", "Show common only", new Message( MSG_SIDEBAR_ATTRIBUTES_SHOW ) ); 
	AddChild( m_Show );

	Layout();
}

AttributesSideBar :: ~AttributesSideBar()
{
	delete m_Links;
	delete m_Show;
}

void AttributesSideBar :: Layout()
{
	Rect cFrame = SideBar::GetFrame();
	ConvertFromParent( &cFrame );

	Point cShow = m_Show->GetPreferredSize( false );
	Rect cShowFrame( cFrame.left, cFrame.bottom - cShow.y, cFrame.right, cFrame.bottom );
	m_Show->SetFrame( cShowFrame );

	Rect cLinksFrame( cFrame.left, cFrame.top, cFrame.right, cShowFrame.top - 5.0f );
	m_Links->SetFrame( cLinksFrame );

}

void AttributesSideBar :: AllAttached( void )
{
	SideBar::AllAttached();

	m_Show->SetTarget( this );
	m_Links->SetTarget( this );
}

void AttributesSideBar :: HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )	//Get the message code from the message
	{		
		case MSG_SIDEBAR_ATTRIBUTES_SHOW:
			break;
	}
}

void AttributesSideBar :: SelectionChanged( vector<String> files )
{
	m_Links->Clear();

	vector<String>::iterator it;

	for( it = files.begin() ; it != files.end() ; it++ )
	{
		try
		{
			String key, value;
			FSNode fn( *it );

			fn.RewindAttrdir();

			while( fn.GetNextAttrName( &key ) == 1 )
			{
				// Read type of attribute
				struct::attr_info ai;
				fn.StatAttr( key, &ai );

				switch( ai.ai_type )
				{
					case ATTR_TYPE_NONE:
					{
						value = "Raw data";
						break;
					}
					case ATTR_TYPE_INT32:
					{
						int32 d;					
						fn.ReadAttr( key, ATTR_TYPE_INT32, &d, 0, ai.ai_size );
						value.Format( "%d", d );
						break;		
					}
					case ATTR_TYPE_INT64:
					{
						int64 d;	
						fn.ReadAttr( key, ATTR_TYPE_INT64, &d, 0, ai.ai_size );
						value.Format( "%d", d );
						break;	
					}
					case ATTR_TYPE_FLOAT:
					{
						float d;					
						fn.ReadAttr( key, ATTR_TYPE_FLOAT, &d, 0, ai.ai_size );
						value.Format( "%f", d );
						break;
					}
					case ATTR_TYPE_DOUBLE:
					{
						double d;					
						fn.ReadAttr( key, ATTR_TYPE_DOUBLE, &d, 0, ai.ai_size );
						value.Format( "%lf", d );
						break;
					}			
					case ATTR_TYPE_STRING:
					{

						char d[ ai.ai_size + 1 ]; 
						fn.ReadAttr( key, ATTR_TYPE_STRING, d, 0, ai.ai_size );
						d[ ai.ai_size ] = '\0';
						value = d;
						break;
					}

					default:
					{
						value = "Unknown type";
						break;
					}
				}

				// Add to the listview
				ListViewStringRow* pcRow = new ListViewStringRow();
				pcRow->AppendString( key );
				pcRow->AppendString( value );
				m_Links->InsertRow( pcRow );			
			}
	
		}
		catch( ... ) { }
	}
}
