// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <util/message.h>
#include <gui/window.h>

#include "sidebar_favourites.h"
#include "messages.h"

using namespace os;

FavouritesSideBar :: FavouritesSideBar( const Rect& cFrame, const String& cTitle, uint32 nResizeMask, uint32 nFlags ) : SideBar( cFrame, cTitle, nResizeMask, nFlags )
{
	m_Links = new ListView( Rect(), "links" );
	m_Links->InsertColumn( "Name", 180 );
	m_Links->SetMultiSelect( false );
	m_Links->SetAutoSort( false );
	m_Links->SetInvokeMsg( new Message( MSG_SIDEBAR_FAVOURITES_SELECT ) );
	AddChild( m_Links );	

	m_Add = new Button( Rect(), "add", "Quick Add", new Message( MSG_SIDEBAR_FAVOURITES_ADD ) ); 
	AddChild( m_Add );

	Layout();
}

FavouritesSideBar :: ~FavouritesSideBar()
{
	delete m_Links;
	delete m_Add;
}

void FavouritesSideBar :: Layout()
{
	Rect cFrame = SideBar::GetFrame();
	ConvertFromParent( &cFrame );

	Point cAdd = m_Add->GetPreferredSize( false );
	Rect cAddFrame( cFrame.right - cAdd.x, cFrame.bottom - cAdd.y, cFrame.right, cFrame.bottom );
	m_Add->SetFrame( cAddFrame );

	Rect cLinksFrame( cFrame.left, cFrame.top, cFrame.right, cAddFrame.top - 5.0f );
	m_Links->SetFrame( cLinksFrame );

}

void FavouritesSideBar :: AllAttached( void )
{
	SideBar::AllAttached();

	m_Add->SetTarget( this );
	m_Links->SetTarget( this );
}

void FavouritesSideBar :: HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )	//Get the message code from the message
	{		
		case MSG_SIDEBAR_FAVOURITES_ADD:
			GetWindow()->HandleMessage( pcMessage );
			break;
		case MSG_SIDEBAR_FAVOURITES_SELECT:
			SelectLink();
			break;
	}
}

void FavouritesSideBar :: Populate( Favourites* favourites )
{
	m_Links->Clear();

	for( int i = 0 ; i < favourites->GetFavouritesCount() ; i ++ )
	{
		String name, path;

		favourites->GetFavourite( i, name, path );

		// Add to the listview
		ListViewStringRow* pcRow = new ListViewStringRow();
		pcRow->AppendString( name );
		pcRow->SetCookie( path );
		m_Links->InsertRow( pcRow );
	}
}

void FavouritesSideBar :: SelectLink()
{
	ListViewRow* row = m_Links->GetRow( m_Links->GetLastSelected() );
	Variant v = row->GetCookie();

	Message msg( MSG_DIRBAR_CHANGE_DIR );
	msg.AddString( "file/path", v.AsString()  );
	GetWindow()->HandleMessage( &msg );
}

