// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <util/message.h>
#include <gui/window.h>

#include "sidebar_search.h"
#include "messages.h"

using namespace os;

SearchSideBar :: SearchSideBar( const Rect& cFrame, const String& cTitle, uint32 nResizeMask, uint32 nFlags ) : SideBar( cFrame, cTitle, nResizeMask, nFlags )
{
	m_LabelFilename = new StringView( Rect(), "label_filename", "Filename filter:" );
	AddChild( m_LabelFilename );

	m_Filename = new TextView( Rect(), "entry_filename", "" );
	m_Filename->SetMultiLine( false );
	AddChild( m_Filename );

	m_LabelContent = new StringView( Rect(), "label_content", "Content:" );
	AddChild( m_LabelContent );

	m_Content = new TextView( Rect(), "entry_content", "" );
	m_Content->SetMultiLine( false );
	AddChild( m_Content );

	m_RecursiveSearch = new CheckBox( Rect(), "checkbox", "Recursive search", new Message( MSG_SIDEBAR_RECURSIVE_SEARCH ) );
	m_RecursiveSearch->SetValue( true );
	AddChild( m_RecursiveSearch );

	m_Search = new Button( Rect(), "search", "Search", new Message( MSG_SIDEBAR_SEARCH ) ); 
	AddChild( m_Search );

	Layout();
}

SearchSideBar :: ~SearchSideBar()
{
	delete m_LabelFilename;
	delete m_LabelContent;
	delete m_Filename;
	delete m_Content;
	delete m_RecursiveSearch;
	delete m_Search;
}

void SearchSideBar :: Layout()
{
	const float offset_x = 8.0f;
	const float offset_y = 25.0f;

	Rect cFrame = GetFrame();

	Point cLabel1 = m_LabelFilename->GetPreferredSize( false );
	Rect cLabel1Frame( offset_x, offset_y, offset_x + cLabel1.x, offset_y + cLabel1.y );
	m_LabelFilename->SetFrame( cLabel1Frame );

	Point cFilename = m_Filename->GetPreferredSize( false );
	Rect cFilenameFrame( offset_x, cLabel1Frame.bottom + 1.0f, cFrame.right - offset_x, cLabel1Frame.bottom + 1.0f + cFilename.y );
	m_Filename->SetFrame( cFilenameFrame );

	Point cLabel2 = m_LabelContent->GetPreferredSize( false );
	Rect cLabel2Frame( offset_x, cFilenameFrame.bottom + 10.0f, offset_x + cLabel2.x, cFilenameFrame.bottom + 10.0f + cLabel2.y );
	m_LabelContent->SetFrame( cLabel2Frame );

	Point cContent = m_Content->GetPreferredSize( false );
	Rect cContentFrame( offset_x, cLabel2Frame.bottom + 1.0f, cFrame.right - offset_x, cLabel2Frame.bottom + 1.0f + cContent.y );
	m_Content->SetFrame( cContentFrame );

	Point cCheckbox = m_RecursiveSearch->GetPreferredSize( false );
	Rect cCheckboxFrame( offset_x, cContentFrame.bottom + 10.0f, cFrame.right - offset_x, cContentFrame.bottom + 10.0f + cCheckbox.y );
	m_RecursiveSearch->SetFrame( cCheckboxFrame );

	Point cSearch = m_Search->GetPreferredSize( false );
	Rect cSearchFrame( cFrame.right - offset_x - cSearch.x, cCheckboxFrame.bottom + 10.0f, cFrame.right - offset_x, cCheckboxFrame.bottom + 10.0f + cSearch.y );
	m_Search->SetFrame( cSearchFrame );
}

void SearchSideBar :: AllAttached( void )
{
	SideBar::AllAttached();

	m_RecursiveSearch->SetTarget( this );
	m_Search->SetTarget( this );
}

void SearchSideBar :: HandleMessage( Message* pcMessage )
{

	switch( pcMessage->GetCode() )	//Get the message code from the message
	{		
		case MSG_SIDEBAR_RECURSIVE_SEARCH:
		break;
		case MSG_SIDEBAR_SEARCH:
			pcMessage->RemoveName( "filename_filter" );
			pcMessage->AddString( "filename_filter", m_Filename->GetValue() );
			pcMessage->RemoveName( "content" );
			pcMessage->AddString( "content", m_Content->GetValue() );
			pcMessage->RemoveName( "recursive" );
			pcMessage->AddBool( "recursive", m_RecursiveSearch->GetValue() );
			GetWindow()->HandleMessage( pcMessage );
			break;
	}
}
