// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "treeviewkeeper.h"
#include <storage/fsnode.h>
#include "sidebar_treeview.h"

using namespace os;
using namespace std;

TreeViewKeeper::TreeViewKeeper( const Messenger& cTarget, const String& cPath ):Looper( "treeview_worker" ), m_cTarget( cTarget )
{
	m_bRunning = false;
	m_pcCurrentDir = NULL;
	try
	{
		m_cPath = cPath;
		m_pcCurrentDir = new Directory( cPath );
		m_pcCurrentDir->Rewind();
		m_cMonitor.SetTo( m_pcCurrentDir, NWATCH_DIR | NWATCH_NAME, this, this );
	} catch(...) { }

	m_eState = S_IDLE;
	m_bWaitForAddReply = false;
	m_bWaitForRemoveReply = false;
	m_bWaitForUpdateReply = false;
}

void TreeViewKeeper::HandleMessage( Message* pcMessage )
{
	switch ( pcMessage->GetCode() )
	{
		case M_CHANGE_DIR:
		{
			String cNewPath;
			pcMessage->FindString( "path", &cNewPath );
			try
			{
				m_cPath = cNewPath;
				Directory *pcNewDir = new Directory( cNewPath );
				delete m_pcCurrentDir;

				m_pcCurrentDir = pcNewDir;
				m_pcCurrentDir->Rewind();
                                
				m_eState = S_READDIR;
				if( !( cNewPath == "/" ) )
					m_cMonitor.SetTo( m_pcCurrentDir, NWATCH_DIR | NWATCH_NAME, this, this );
				else
					m_cMonitor.Unset();

				m_cAddMap.clear();
				m_cRemoveMap.clear();
				m_cUpdateMap.clear();
				m_bWaitForAddReply = false;
				m_bWaitForRemoveReply = false;
				m_bWaitForUpdateReply = false;
			} catch( ... ) { }
			break;
		}
		case M_ENTRIES_ADDED:
		{
			m_bWaitForAddReply = false;
			if( m_cAddMap.empty() == false )
			{
				Message cMsg( TreeViewSideBar::M_ADD_ENTRY );
				int nCount = 0;

				while( m_cAddMap.empty() == false )
				{
					vector < FileNode >::iterator i = m_cAddMap.begin();

					FSNode cNode;

					cMsg.AddString( "name", ( *i ).m_cName );
					cMsg.AddString( "path", ( *i ).m_cPath );
					cMsg.AddPointer( "node_monitor", ( *i ).m_pcMonitor );

					m_cAddMap.erase( i );
					if( ++nCount > 5 )
					{
						break;
					}
				}
				m_bWaitForAddReply = true;
				m_cTarget.SendMessage( &cMsg );
			}
			break;
		}
        default:
                Looper::HandleMessage( pcMessage );
                break;
	}
}

void TreeViewKeeper::SendAddMsg( const String& cName )
{
	try
	{
		FSNode cNode;
		NodeMonitor* pcMonitor = NULL;  

		struct stat sStat;

		cNode.GetStat( &sStat, false );
                
		FileNode cFileNode( cName, cName, pcMonitor );
                
                
		if( m_bWaitForAddReply )
		{
			m_cAddMap.push_back( cFileNode );
		}
		else
		{
			Message cMsg( TreeViewSideBar::M_ADD_ENTRY );
			cMsg.AddString( "name", cName );
			cMsg.AddString( "path", cName );
			cMsg.AddPointer( "node_monitor", pcMonitor );

			m_cTarget.SendMessage( &cMsg );
			m_bWaitForAddReply = true;
		}
	}
	catch( ... ) {  }
}


bool TreeViewKeeper::Idle()
{
	switch ( m_eState )
	{
		case S_READDIR:
		{
			if( m_pcCurrentDir == NULL )
			{
				m_eState = S_IDLE;
				return ( true );
			}
			os::String cName;
                        
			if( m_cPath == "/" )
			{
			} 
			else
			{
				/* Show files */
				if( m_pcCurrentDir->GetNextEntry( &cName ) == 1 )
				{
					if( !( cName == "." ) && !( cName == ".." ) )
					{
						SendAddMsg( cName );
					}
					return ( true );
				}
				else
				{
					m_eState = S_IDLE;
					return ( true );
				}
			}
			break;
		}
        case S_IDLE:	
                return ( false );
	}
    
	return ( false );
}


