// EFileBrowser	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef __TREEVIEWKEEPER_H__
#define __TREEVIEWKEEPER_H__

#include <vector>
#include <util/string.h>
#include <util/messenger.h>
#include <util/message.h>
#include <util/looper.h>
#include <storage/directory.h>
#include <storage/nodemonitor.h>

class TreeViewKeeper:public os::Looper
{
public:
	enum { M_CHANGE_DIR = 1, M_COPY_FILES, M_MOVE_FILES, M_DELETE_FILES, M_RENAME_FILES, M_ENTRIES_ADDED, M_ENTRIES_REMOVED, M_ENTRIES_UPDATED };
	TreeViewKeeper( const os::Messenger& cTarget, const os::String& cPath );
	virtual void HandleMessage( os::Message* pcMessage );
	virtual bool Idle();	
	os::Directory* GetCurrentDir() { return( m_pcCurrentDir ); }

private:
	void SendAddMsg( const os::String& cName );
	void SendRemoveMsg( const os::String& cName, dev_t nDevice, ino_t nInode );
	void SendUpdateMsg( const os::String& cName, dev_t nDevice, ino_t nInode, bool bReloadIcon );
	void SendDriveAddMsg( const os::String& cName, const os::String& cPath );

	enum state_t  { S_IDLE, S_READDIR };
	bool m_bRunning;
	os::Messenger m_cTarget;
	os::String m_cPath;
	os::Directory *m_pcCurrentDir;
	os::NodeMonitor m_cMonitor;
	state_t m_eState;
	bool m_bWaitForAddReply;
	bool m_bWaitForRemoveReply;
	bool m_bWaitForUpdateReply;

	struct FileNode	
	{
		FileNode( const os::String& cName, const os::String& cPath, os::NodeMonitor* pcMonitor = NULL ):
				  m_cName( cName ), m_cPath( cPath ), m_pcMonitor( pcMonitor )
		{
		}

		mutable os::String m_cName;
		mutable os::String m_cPath;
		os::NodeMonitor* m_pcMonitor;
	};

	std::vector < FileNode > m_cAddMap;
	std::vector < FileNode > m_cUpdateMap;
	std::vector < FileNode > m_cRemoveMap;
};

#endif

